/**
 * You may contact the author by email: peter.tribble@gmail.com
 *
 * ContentsFileTree - display local files
 *
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.io.File;


public class ContentsFileTree extends JTree {

    private final Cursor cbusy = new Cursor(Cursor.WAIT_CURSOR);
    private final Cursor cdone = new Cursor(Cursor.DEFAULT_CURSOR);

    private DefaultTreeModel model;

    /**
     * Create a JTree representing local files and directories.
     */
    public ContentsFileTree() {
	this("/");
    }

    /**
     * Create a JTree representing local files and directories.
     *
     * @param root   The root of this tree
     */
    public ContentsFileTree(String root) {
	model = createTreeModel(root);

	setModel(model);
	createTreeModelListener();
	this.getSelectionModel().setSelectionMode
	    (TreeSelectionModel.SINGLE_TREE_SELECTION);

	addTreeExpansionListener(new TreeExpansionListener() {
	    public void treeCollapsed(TreeExpansionEvent e) {
	    }
	    public void treeExpanded(TreeExpansionEvent e) {
		TreePath path = e.getPath();
		if (path != null) {
		    ContentsFileTreeNode node =
			(ContentsFileTreeNode) path.getLastPathComponent();

		    if (!node.isExplored()) {
			model = (DefaultTreeModel) getModel();
			setCursor(cbusy);
			node.explore();
			setCursor(cdone);
			model.nodeStructureChanged(node);
		    }
		}
	    }
	});

    }

    public DefaultTreeModel createTreeModel(String dir) {
	ContentsFileTreeNode rootNode = new ContentsFileTreeNode(new File(dir));
	rootNode.explore();
	return new DefaultTreeModel(rootNode);
    }

}
