/**
 * InstalledPackagePanel
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import uk.co.petertribble.jingle.TableSorter;

import org.jdesktop.swingworker.SwingWorker;

/**
 * Show a table inside a Scroll pane, displaying the packages installed.
 * The columns can be sorted by clicking the column headers. Selecting
 * a row causes information to appear in a tabbed area below the table.
 */
public class InstalledPackagePanel extends JPanel {

    private PackageInformationPanel pip;
    private PackageTableModel ptm;

    private SolarisPackage currentPackage = null;

    private PkgList plist;
    private ClusterToc ctoc;

    private ContentsParser cp;

    public InstalledPackagePanel(PkgList plist, ClusterToc ctoc) {
	this.plist = plist;
	this.ctoc = ctoc;

	setLayout(new BorderLayout());

	JPanel jpp = new JPanel(new BorderLayout());
	ptm = new PackageTableModel(plist);
	final TableSorter sortedModel = new TableSorter(ptm);
	final JTable ptable = new JTable(sortedModel);
	jpp.add(new JScrollPane(ptable));
	sortedModel.setTableHeader(ptable.getTableHeader());

	pip = new PackageInformationPanel(true, true, ctoc.exists(), false,
					false);

	JSplitPane psplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
		jpp, pip);
	psplit.setOneTouchExpandable(true);
	psplit.setDividerLocation(180);
	add(psplit);

	ptable.getSelectionModel().addListSelectionListener(
		new ListSelectionListener() {
	    public void valueChanged(ListSelectionEvent e) {
                if (e.getSource() instanceof DefaultListSelectionModel) {
                    if (!e.getValueIsAdjusting()) {
			int irow = ptable.getSelectedRow();
			if (irow >= 0) {
			    // sorted table, need to convert the index
			    int imod = sortedModel.modelIndex(irow);
			    SolarisPackage pkg = ptm.getPackageAtRow(imod);
			    setPackageInfo(pkg);
			}
		    }
		}
	    }
	});
    }

    public void setPackageInfo(SolarisPackage pkg) {
	if (pkg.exists()) {
	    currentPackage = pkg;
	    pip.setInfoText(PkgUtils.infoTable(pkg),
		    PkgUtils.dependencyTable(pkg, plist),
		    plist.revDeps(pkg.getName()));
	    pip.setFilesText(PkgUtils.detailTable(pkg, cp));
	} else {
	    pip.setInfoText("Not installed");
	}
	pip.setClusterText(PkgUtils.clusterMembership(pkg, ctoc));
    }

    public void createRevDependencies() {
	(new RevDependencyWorker()).execute();
    }

    class RevDependencyWorker extends SwingWorker <String, Object> {
	@Override
	public String doInBackground() {
	    plist.createRevDependencies();
	    return "done";
	}

	@Override
	protected void done() {
	    try {
		pip.showRevDependencies();
		if (currentPackage != null) {
		    setPackageInfo(currentPackage);
		}
	    } catch (Exception ignore) {
	    }
	}
    }

    public void showDetailedView() {
	cp = ContentsParser.getInstance();
    }
}
