/**
 * InstalledSoftwarePanel
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import javax.swing.*;

import org.jdesktop.swingworker.SwingWorker;

/**
 * Generates a view of the software installed on a system. The display
 * is broken into three tabs, which show a list of installed packages,
 * a tree view based on installation clusters, and (optionally) a view
 * based on the files in the filesystem.
 */
public class InstalledSoftwarePanel extends JTabbedPane {

    private ClusterToc ctoc;
    private InstalledPackagePanel ipp;
    private InstalledClusterPanel icp;
    private InstalledFilesPanel ifp;

    public InstalledSoftwarePanel() {
	PkgResources pkgres = PkgResources.getInstance();

	PkgList plist = new PkgList();
	ctoc = new ClusterToc();

	ipp = new InstalledPackagePanel(plist, ctoc);
	add(pkgres.getString("PKG.LIST"), ipp);

	if (ctoc.exists()) {
	    icp = new InstalledClusterPanel(plist, ctoc);
	    add(pkgres.getString("PKG.TREE"), icp);
	}

	ifp = new InstalledFilesPanel(ctoc);
	add(pkgres.getString("PKG.FS"), ifp);

	PatchPanel patchp = new PatchPanel(plist);
	add(pkgres.getString("PKG.PATCH"), patchp);

	createRevDependencies();
	(new ContentsWorker()).execute();
    }

    /*
     * Parse the contents file in the background. ContentsParser is
     * a singleton, so once we've done it here we can tell the other
     * views to use it.
     */
    class ContentsWorker extends SwingWorker <String, Object> {
	@Override
	public String doInBackground() {
	    ContentsParser cp = ContentsParser.getInstance();
	    return "done";
	}

	@Override
	protected void done() {
	    try {
		ipp.showDetailedView();
		if (ctoc.exists()) {
		    icp.showDetailedView();
		}
		ifp.showDetailedView();
	    } catch (Exception ignore) {
	    }
	}
    }

    private void createRevDependencies() {
	ipp.createRevDependencies();
	if (ctoc.exists()) {
	    icp.createRevDependencies();
	}
    }
}
