/**
 * InfoCommandList - a List of Solaris information commands
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview;

import java.util.*;

public class InfoCommandList extends Vector <InfoCommand> {

    // a backing hash so we can look up commands by description
    private Map <String, InfoCommand> infoMap;

    private SolViewResources svr;

    /**
     * Construct a List of available commands.
     * Note that it's actually a Vector.
     */
    public InfoCommandList() {
	svr = SolViewResources.getInstance();

	infoMap = new HashMap <String, InfoCommand> ();
	// add all the possible commands here
	addCommand(svr.getString("INFO.CORE"), "/usr/bin/coreadm");
	addCommand(svr.getString("INFO.DF"), "/usr/sbin/df", "-kl");
	addCommand(svr.getString("INFO.IOSTAT"), "/usr/bin/iostat", "-En");
	addCommand(svr.getString("INFO.HOSTID"), "/usr/bin/hostid");
	addCommand(svr.getString("INFO.IFCONFIG"), "/usr/sbin/ifconfig", "-a");
	addCommand(svr.getString("INFO.INETADM"), "/usr/sbin/inetadm");
	addCommand(svr.getString("INFO.ISA"), "/usr/bin/isainfo", "-v");
	addCommand(svr.getString("INFO.META"), "/usr/sbin/metastat");
	addCommand(svr.getString("INFO.MDDB"), "/usr/sbin/metadb", "-i");
	addCommand(svr.getString("INFO.MOD"), "/usr/sbin/modinfo");
	addCommand(svr.getString("INFO.NSCD"), "/usr/sbin/nscd", "-g");
	addCommand(svr.getString("INFO.NTP"), "/usr/sbin/ntpq", "-p");
	addCommand(svr.getString("INFO.PKGS"), "/usr/bin/pkginfo");
	addCommand(svr.getString("INFO.PATCH"), "/usr/bin/showrev", "-p");
	addCommand(svr.getString("INFO.EEPROM"), "/usr/sbin/eeprom");
	addCommand(svr.getString("INFO.PRTCONF"), "/usr/sbin/prtconf");
	addCommand(svr.getString("INFO.PRTDIAG"), "/usr/sbin/prtdiag");
	addCommand(svr.getString("INFO.PRTFRU"), "/usr/sbin/prtfru");
	addCommand(svr.getString("INFO.PRTPICL"), "/usr/sbin/prtpicl");
	addCommand(svr.getString("INFO.PSRINFO"), "/usr/sbin/psrinfo", "-v");
	addCommand(svr.getString("INFO.ROUTE"), "/sbin/routeadm");
	addCommand(svr.getString("INFO.RPC"), "/usr/bin/rpcinfo", "-s");
	addCommand(svr.getString("INFO.SHARE"), "/usr/sbin/share");
	addCommand(svr.getString("INFO.SMBIOS"), "/usr/sbin/smbios");
	addCommand(svr.getString("INFO.SWAP"), "/usr/sbin/swap", "-l");
	addCommand(svr.getString("INFO.UNAME"), "/usr/bin/uname", "-a");
	addCommand(svr.getString("INFO.UP"), "/usr/bin/uptime");
	addCommand(svr.getString("INFO.ZPOOL"), "/usr/sbin/zpool", "status");
	addCommand(svr.getString("INFO.ZFS"), "/usr/sbin/zfs", "list");
	addCommand(svr.getString("INFO.ZONES"),
		"/usr/sbin/zoneadm", "list -icv");
    }

    private void addCommand(String text, String cmd) {
	addCommand(text, cmd, (String) null);
    }

    private void addCommand(String text, String cmd, String args) {
	addCommand(new InfoCommand(text, cmd, args));
    }

    private void addCommand(InfoCommand infocmd) {
	if (infocmd.exists() &&
			!infoMap.containsKey(infocmd.toString())) {
	    infoMap.put(infocmd.toString(), infocmd);
	    add(infocmd);
	}
    }

    public InfoCommand getCmd(String text) {
	return infoMap.get(text);
    }

    public String infoTable(String pkg) {
	return infoTable(infoMap.get(pkg));
    }

    public String infoTable(InfoCommand infocmd) {
	StringBuilder sb = new StringBuilder();
	sb.append(svr.getString("INFO.OUTPUT.TEXT"));
	sb.append(" <b>").append(infocmd.getFullCmd());
	sb.append("</b>\n<hr>\n<pre>");
	sb.append(infocmd.getOutput());
	sb.append("</pre>");
	return sb.toString();
    }
}
