/**
 * SolView - shows Solaris information
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import uk.co.petertribble.jingle.JingleMultiFrame;
import uk.co.petertribble.jingle.JingleInfoFrame;
import uk.co.petertribble.pkgview.InstalledSoftwarePanel;
import uk.co.petertribble.pkgview.DistributionSoftwarePanel;
import uk.co.petertribble.solview.explorer.SysPanel;
import uk.co.petertribble.jkstat.api.JKstat;
import uk.co.petertribble.jkstat.demo.KstatToolsMenu;

public class SolView implements ActionListener {

    private static String[] savedArgs;

    private JMenuItem exitItem;
    private JMenuItem cloneItem;
    private JMenuItem closeItem;
    private JMenuItem helpItem;
    private JMenuItem licenseItem;
    private JFrame jf;

    public SolView(String[] args) {

	SolViewResources svr = SolViewResources.getInstance();

	jf = new JFrame("Solview");
	jf.addWindowListener(new winExit());

	JTabbedPane jtp = new JTabbedPane();
	jf.getContentPane().add(jtp, BorderLayout.CENTER);

	JMenuBar jm = new JMenuBar();

	jtp.add(svr.getString("SOLVIEW.SOFT.TEXT"),
		new InstalledSoftwarePanel());
	jtp.add(svr.getString("SOLVIEW.SERV.TEXT"), new SmfPanel());
	jtp.add(svr.getString("SOLVIEW.INFO.TEXT"), new InfoPanel());
	jtp.add(svr.getString("SOLVIEW.EXPL.TEXT"), new SysPanel());

	JMenu jme = new JMenu(svr.getString("FILE.TEXT"));
	jme.setMnemonic(KeyEvent.VK_F);
	cloneItem = new JMenuItem(svr.getString("FILE.NEWBROWSER.TEXT"),
				KeyEvent.VK_B);
	cloneItem.addActionListener(this);
	jme.add(cloneItem);
	closeItem = new JMenuItem(svr.getString("FILE.CLOSEWIN.TEXT"),
				KeyEvent.VK_W);
	closeItem.addActionListener(this);
	jme.add(closeItem);
	exitItem = new JMenuItem(svr.getString("FILE.EXIT.TEXT"),
				KeyEvent.VK_X);
	exitItem.addActionListener(this);
	jme.add(exitItem);

	JingleMultiFrame.register(jf, closeItem);

	jm.add(jme);
	jm.add(new SolToolsMenu());

	/*
	 * Conditionally add a menu of Kstat tools
	 */
	try {
	    JKstat jkstat = new JKstat();
	    jm.add(new KstatToolsMenu(jkstat));
	} catch (Exception e) {
	    // ignored
	}

	JMenu jmh = new JMenu(svr.getString("HELP.TEXT"));
	jmh.setMnemonic(KeyEvent.VK_H);
	helpItem = new JMenuItem(svr.getString("HELP.ABOUT.TEXT") + " solview",
				KeyEvent.VK_A);
	helpItem.addActionListener(this);
	jmh.add(helpItem);
	licenseItem = new JMenuItem(svr.getString("HELP.LICENSE.TEXT"),
				KeyEvent.VK_L);
	licenseItem.addActionListener(this);
	jmh.add(licenseItem);

	jm.add(jmh);
	jf.setJMenuBar(jm);

	for (int i = 0; i < args.length; i++) {
	    DistributionSoftwarePanel dsp =
		new DistributionSoftwarePanel(args[i]);
	    jtp.add(args[i], dsp);
	}

	jf.setSize(720, 600);
	jf.setVisible(true);
    }

    class winExit extends WindowAdapter {
	public void windowClosing(WindowEvent we) {
	    JingleMultiFrame.unregister(jf);
	}
    }

    public void actionPerformed(ActionEvent e) {
	if (e.getSource() == cloneItem) {
	    new SolView(savedArgs);
	}
	if (e.getSource() == closeItem) {
	    JingleMultiFrame.unregister(jf);
	}
	if (e.getSource() == exitItem) {
	    System.exit(0);
	}
	if (e.getSource() == helpItem) {
	    new JingleInfoFrame(this.getClass().getClassLoader(),
				"help/index.html", "text/html");
	}
	if (e.getSource() == licenseItem) {
	    new JingleInfoFrame(this.getClass().getClassLoader(),
				"help/CDDL.txt", "text/plain");
	}
    }

    public static void main(String args[]) {
	savedArgs = args;
	new SolView(args);
    }

}
