/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package uk.co.petertribble.solview.explorer;

import javax.swing.*;
import java.awt.event.*;
import uk.co.petertribble.jkstat.api.JKstat;
import uk.co.petertribble.jkstat.demo.JKdemo;
import uk.co.petertribble.solview.SolViewResources;

public class JFSstat extends JKdemo implements ActionListener {

    private static final String sversion = "JFSstat version 0.45";

    private JFSstatPanel mainPanel;

    public JFSstat() {
	this(new JKstat(), true);
    }

    public JFSstat(JKstat jkstat, boolean standalone) {
	super("jfsstat", standalone);

	mainPanel = new JFSstatPanel(jkstat, DEFAULT_INTERVAL);

	setContentPane(mainPanel);

	addInfoPanel(mainPanel, sversion);

	JMenu showMenu = new JMenu(SolViewResources.getString(
							"FSSTAT.SHOW.TEXT"));
	String curtitle = mainPanel.getCurrentTitle();
	ButtonGroup showGroup = new ButtonGroup();
	for (String title : mainPanel.getTitles()) {
	    JRadioButtonMenuItem jmis =
		new JRadioButtonMenuItem(title, title.equals(curtitle));
	    showMenu.add(jmis);
	    showGroup.add(jmis);
	    jmis.addActionListener(this);
	}
	addMenu(showMenu);

	setSize(620, 250);
	validate();
	setVisible(true);
    }

    public void actionPerformed(ActionEvent e) {
	super.actionPerformed(e);
	JMenuItem jmi = (JMenuItem) e.getSource();
	mainPanel.setNames(jmi.getText());
    }

    @Override
    public void stopLoop() {
	mainPanel.stopLoop();
    }

    @Override
    public void setDelay(int i) {
	mainPanel.setDelay(i);
	setLabelDelay(i);
    }

    public static void main(String[] args) {
	new JFSstat();
    }
}
