/**
 * SysItem - represent an item of hardware
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview.explorer;

import java.util.Map;
import java.util.HashMap;

public class SysItem {

    public static final int LABEL = 1;

    public static final int CPU_CONTAINER = 11;
    public static final int CPU = 12;
    public static final int CPU_CORE = 13;
    public static final int CPU_THREAD = 14;

    public static final int DISK_CONTAINER = 21;
    public static final int DISK = 22;
    public static final int DISK_PARTITION = 23;
    public static final int DISK_METADEVICE = 24;
    public static final int DISK_META_CONTAINER = 25;

    public static final int NET_CONTAINER = 31;
    public static final int NET_INTERFACE = 32;
    public static final int NET_PROTOCOL = 33;
    public static final int NET_PROTO_IP = 34;
    public static final int NET_PROTO_TCP = 35;
    public static final int NET_PROTO_UDP = 36;

    public static final int MEM_CONTAINER = 41;
    public static final int MEM_KMEM = 42;
    public static final int MEM_ARCSTAT = 43;

    public static final int FS_CONTAINER = 51;
    public static final int FS_FSSTAT = 52;
    public static final int ZFS_CONTAINER = 53;
    public static final int ZFS_POOL = 54;
    public static final int ZFS_FS = 55;

    private Map <String, Object> attributes;
    private int type;
    private int instance;

    /**
     * A container for a hardware item. Needs to be expanded.
     */
    public SysItem(int type) {
	this(type, 0);
    }

    /**
     * A container for a hardware item. Needs to be expanded.
     */
    public SysItem(int type, int instance) {
	this.type = type;
	this.instance = instance;
	attributes = new HashMap <String, Object> ();
    }

    public int getType() {
	return type;
    }

    public int getInstance() {
	return instance;
    }

    public void addAttribute(String key, Object value) {
	attributes.put(key, value);
    }

    public Object getAttribute(String key) {
	return attributes.get(key);
    }
}
