/**
 * Zfilesys - represent a ZFS filesystem
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview.explorer;

import java.util.Map;
import java.util.HashMap;
import uk.co.petertribble.solview.InfoCommand;

public class Zfilesys {

    private String name;
    private Map <String, String> propmap;
    private boolean readprops = false;

    // FIXME just create based on name; then use 'zfs list -Hp name'
    // to fill in the rest of the proerties

    public Zfilesys(String name) {
	this.name = name;
	propmap = new HashMap <String, String> ();
	// FIXME should we automatically do this?
	// may not scale well!
	// setProperties();
    }

    /*
     * This assumes we're just interested in the property name and its value.
     * If we want to track the source, then we'll need to define a property
     * object.
     */
    private void setProperties() {
	readprops = true;
	InfoCommand ic = new InfoCommand("ZP", "/usr/sbin/zfs",
						"get -Hp all " + name);
	if (ic.exists()) {
	    String[] lines = ic.getOutput().split("\n");
	    for (int i = 0; i < lines.length; i++) {
		String[] ds = lines[i].split("\\s+");
		propmap.put(ds[1], ds[2]);
	    }
	}
    }

    /**
     * Return the name of this filesystem.
     */
    public String getName() {
	return name;
    }

    /**
     * Refresh the list of zfs properties for this filesystem.
     */
    public void update() {
	setProperties();
    }

    /**
     * Return the entire property map
     */
    public Map <String, String> getProperties() {
	if (!readprops) {
	    setProperties();
	}
	return propmap;
    }

    /**
     * Retrieve the value of the specified property.
     */
    public String getProperty(String key) {
	if (!readprops) {
	    setProperties();
	}
	return propmap.get(key);
    }

    /**
     * Return the String representation of this ZFS filesystem, its name.
     */
    public String toString() {
	return name;
    }
}
